package corefdb

import (
	"log"
	"path/filepath"
	"testing"
	"time"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/dalmdl/lazywriter"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/filemdl/filepack"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/filemdl"
	"github.com/tidwall/buntdb"
	"github.com/tidwall/gjson"
	"github.com/tidwall/sjson"

	"corelab.mkcl.org/MKCLOS/coredevelopmentplatform/corepkgv2/loggermdl"
)

var dbInstance *FDB
var bucketInstance *Bucket
var indexInstance *Index

func init() {
	Init(false, false, false, "")

	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}

	dbInstance = db
	// step 1:  create bucket
	bucket := db.GetNewBucket("Candidates", false, &Bucket{})
	bucketInstance = bucket
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(bucket)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
		IndexField{
			FieldName: "class",
			Query:     "class",
		},
	}
	i.SetFields(fields...)
	indexInstance = i
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestSaveDataInNormalBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "name", "ajay")
	data, _ = sjson.Set(data, "studentId", 1237)
	data, _ = sjson.Set(data, "examName", "unit2")
	data, _ = sjson.Set(data, "totalQuestion", 50)
	data, _ = sjson.Set(data, "marks", 26)
	data, _ = sjson.Set(data, "examId", "MATH001")
	data, _ = sjson.Set(data, "fileType", "Exam")

	studentObj := gjson.Parse(data)
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
	if err != nil {
		log.Fatal(err)
	}

	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestGetDataFromNormalBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/importedFDB", "importedFDB", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	//   create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	loggermdl.LogError(indexFilePath)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	// queries := []string{`#[name==ajay]`}

	data, _ := sjson.Set("", "fileType", "Exam")
	inFileIndexQueries := []string{`#[examId=="MATH001"]`}

	// data, _ := sjson.Set("", "fileType", "Profile")
	// inFileIndexQueries := []string{`#[class==TY_MCA]`}

	queries := []string{`#[name==ajay]`}
	// inFileIndexQueries := []string{`#[class==SY_MCA]`}

	studentObj := gjson.Parse(data)
	// step 3: save data
	result, err := ReadDataFromFDB("importedFDB", i.IndexID, &studentObj, queries, inFileIndexQueries)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("result", result.String())
}

func TestUpdateDataInNormalBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "abc", 10)
	data, _ = sjson.Set(data, "marks", 3)
	// data, _ = sjson.Set(data, "fileType", "Exam")

	queries := []string{`#[name=="ajay"]`}
	infileIndexQueries := []string{}
	studentObj := gjson.Parse(data)
	// step 3: save data
	updatedData, errList := UpdateDataInFDB("myfdb", i.IndexID, &studentObj, queries, infileIndexQueries)
	if len(errList) > 0 {
		loggermdl.LogError(errList)
	}
	loggermdl.LogDebug("updatedData", updatedData)
}

func TestDeleteDataFromNormalBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	queries := []string{`#[name=="ajay"]`}
	infileIndexQueries := []string{}

	// data, _ := sjson.Set("", "fileType", "Exam")
	// data, _ := sjson.Set(data, "studentId", 1234)

	studentObj := gjson.Result{}

	recordsDeletedCnt, errList := DeleteDataFromFDB("myfdb", i.IndexID, &studentObj, queries, infileIndexQueries)
	if len(errList) > 0 {
		loggermdl.LogError(errList)
	}
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("recordsDeletedCnt", recordsDeletedCnt)

}

func TestSaveDataInPackBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "examId",
				Query:     "examId",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("stdId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	// data, _ := sjson.Set("", "name", "ajay")
	// data, _ = sjson.Set(data, "studentId", 10013)
	// data, _ = sjson.Set(data, "class", "TY_MCA")
	// data, _ = sjson.Set(data, "age", 24)
	// data, _ = sjson.Set(data, "fileType", "Profile")

	data, _ := sjson.Set("", "name", "ajay")
	data, _ = sjson.Set(data, "stdId", 1237)
	data, _ = sjson.Set(data, "examName", "unit2")
	data, _ = sjson.Set(data, "totalQuestion", 50)
	data, _ = sjson.Set(data, "marks", 26)
	data, _ = sjson.Set(data, "examId", "MATH001")
	data, _ = sjson.Set(data, "fileType", "Exam")

	//////// ------------vijay

	// data, _ := sjson.Set("", "name", "vijay")
	// data, _ = sjson.Set(data, "studentId", 10014)
	// data, _ = sjson.Set(data, "class", "SY_MCA")
	// data, _ = sjson.Set(data, "age", 23)
	// data, _ = sjson.Set(data, "fileType", "Profile")

	// data, _ := sjson.Set("", "studentId", 10014)
	// data, _ = sjson.Set(data, "name", "vijay")
	// data, _ = sjson.Set(data, "examName", "Unit3")
	// data, _ = sjson.Set(data, "totalQuestion", 100)
	// data, _ = sjson.Set(data, "marks", 32)
	// data, _ = sjson.Set(data, "fileType", "Exam")

	// studentList := []gjson.Result{}
	// for index := 0; index < 5; index++ {
	// 	data, _ = sjson.Set(data, "studentId", index+1)
	// 	studentList = append(studentList, gjson.Parse(data))
	// }
	// for _, studentObj := range studentList {
	// 	err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
	// 	if err != nil {
	// 		log.Fatal(err)
	// 	}
	// }
	studentObj := gjson.Parse(data)

	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("loaded")
	err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("saved")
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestGetDataFromPackBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "examId",
				Query:     "examId",
			},
		},
	}
	inFileFDBIndex := filepack.InFileIndex{
		FileType: "FDBIndex",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "name",
				Query:     "name",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	b.SetInFileIndex(inFileFDBIndex)
	i, err := db.GetNewIndex("stdId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	//   create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	// queries := []string{`#[name==ajay]`}

	data, _ := sjson.Set("", "fileType", "Exam")
	inFileIndexQueries := []string{`#[examId=="MATH001"]`}

	// data, _ := sjson.Set("", "fileType", "Profile")
	// inFileIndexQueries := []string{`#[class==TY_MCA]`}

	queries := []string{`#[name=="ajay"]`}
	// inFileIndexQueries := []string{`#[class==SY_MCA]`}

	studentObj := gjson.Parse(data)
	// step 3: save data
	result, err := ReadDataFromFDB("myfdb", i.IndexID, &studentObj, queries, inFileIndexQueries)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("result", result)
}

func TestUpdateDataInPackBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}

	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "marks",
				Query:     "marks",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("stdId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	// data, _ := sjson.Set("", "name", "ajay")
	// data, _ = sjson.Set(data, "studentId", 10013)
	// data, _ = sjson.Set(data, "class", "TY_MCA")
	// data, _ = sjson.Set(data, "age", 24)
	// data, _ = sjson.Set(data, "fileType", "Profile")

	// data, _ := sjson.Set("", "studentId", 10013)
	// data, _ = sjson.Set(data, "name", "ajay")
	// data, _ = sjson.Set(data, "examName", "Unit1")
	// data, _ = sjson.Set(data, "totalQuestion", 50)
	// data, _ = sjson.Set(data, "marks", 30)
	// data, _ = sjson.Set(data, "fileType", "Exam")

	//////// ------------ajay

	// data, _ := sjson.Set("", "name", "vijay")
	// data, _ = sjson.Set(data, "studentId", 10014)
	// data, _ = sjson.Set(data, "class", "SY_MCA")
	// data, _ = sjson.Set(data, "age", 23)
	// data, _ = sjson.Set(data, "fileType", "Profile")

	// data, _ := sjson.Set("", "studentId", 10014)
	// data, _ = sjson.Set(data, "name", "vijay")
	// data, _ = sjson.Set(data, "examName", "t")
	data, _ := sjson.Set("", "abc", 150)
	data, _ = sjson.Set(data, "marks", 32)
	data, _ = sjson.Set(data, "fileType", "Exam")

	queries := []string{`#[name=="ajay"]`}
	infileIndexQueries := []string{`#[examId=="MATH001"]`}
	studentObj := gjson.Parse(data)
	// step 3: save data
	updatedData, errList := UpdateDataInFDB("myfdb", i.IndexID, &studentObj, queries, infileIndexQueries)
	if len(errList) > 0 {
		loggermdl.LogError(errList)
	}
	loggermdl.LogDebug("updatedData", updatedData)
}

func TestDeleteDataFromPackBucket(t *testing.T) {
	// TestSaveDataInFDB(t)

	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "examId",
				Query:     "examId",
			},
		},
	}
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("stdId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	data, _ := sjson.Set("", "fileType", "Exam")

	// data, _ = sjson.Set(data, "studentId", 1234)

	// studentObj := gjson.Result{}

	// err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
	// if err != nil {
	// 	log.Fatal(err)
	// }

	queries := []string{`#[name=="ajay"]`}
	infileIndexQueries := []string{`#[examId=="MATH001"]`}
	studentObj := gjson.Parse(data)
	recordsDeleted, errList := DeleteDataFromFDB("myfdb", i.IndexID, &studentObj, queries, infileIndexQueries)
	if len(errList) > 0 {
		loggermdl.LogError(errList)
	}
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("recordsDeleted", recordsDeleted)
}

func TestReindexOnPackBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexFDB := filepack.InFileIndex{
		FileType: "FDBIndex",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "name",
				Query:     "name",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexFDB)
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = ReindexOnSpecialBucket("myfdb", i.IndexID, gjson.Result{})
	if err != nil {
		log.Fatal(err)
	}

}

func TestReindexOnSimpleBucket(t *testing.T) {

	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Candidates", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
		IndexField{
			FieldName: "class",
			Query:     "class",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "name", "vivek")
	data, _ = sjson.Set(data, "studentId", 1000)

	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 26)

	studentObj := gjson.Parse(data)
	// step 3: save data
	err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
	if err != nil {
		log.Fatal(err)
	}
	// step 4: fetch data
	result, err := ReadDataFromFDB("myfdb", i.IndexID, &studentObj, []string{`#[name=="vivek"]`}, []string{""})
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("before reindex", result)

	// step 5: delete index file
	err = i.indexObj.Close()
	if err != nil {
		log.Fatal(err)
	}

	err = filemdl.DeleteFile("/home/vivekn/fdb_data/myfdb/index/studentId")
	if err != nil {
		loggermdl.LogError(err)
		log.Fatal(err)
	}

	// step 6: reindex
	err = db.ReIndex(i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	// step 7: fetch data
	data2, err := ReadDataFromFDB("myfdb", i.IndexID, &studentObj, []string{`#[name=="vivek"]`}, []string{""})

	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("after reindex", data2)
}

func TestLogFDBIndexFile(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "marks",
				Query:     "marks",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "name", "ajay")
	data, _ = sjson.Set(data, "examName", "unit2")
	data, _ = sjson.Set(data, "totalQuestion", 50)
	data, _ = sjson.Set(data, "marks", 26)
	data, _ = sjson.Set(data, "fileType", "Exam")

	studentList := []gjson.Result{}
	for index := 0; index < 1; index++ {
		data, _ = sjson.Set(data, "studentId", index+1)
		studentList = append(studentList, gjson.Parse(data))
	}
	// step 3: save data
	for _, studentObj := range studentList {
		// data, _ = sjson.Set(data, "studentId", v+1)
		// studentObj := gjson.Parse(data)
		err = SaveDataInFDB("myfdb", i.IndexID, &studentObj)
		if err != nil {
			log.Fatal(err)
		}
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}

}

func TestLoadFDBIndexFromFile(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackType", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "marks",
				Query:     "marks",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "fileType", "Exam")
	inFileIndexQueries := []string{`#[marks==26]`}

	// data, _ := sjson.Set("", "fileType", "Profile")
	// inFileIndexQueries := []string{`#[class==TY_MCA]`}

	queries := []string{`#[name=="ajay"]`}
	// inFileIndexQueries := []string{`#[class==SY_MCA]`}

	studentObj := gjson.Parse(data)
	// step 3: save data
	result, err := ReadDataFromFDB("myfdb", i.IndexID, &studentObj, queries, inFileIndexQueries)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug("result", result)
}
func TestLazyAppend(t *testing.T) {
	// Init(false, false, false, "")
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("appendBucket", false, &Bucket{})
	b.SetBucketType(BucketTypeAppend)
	b.EnableLazyWrite(true)
	// loggermdl.LogDebug(infileIndexExam)
	i, err := db.GetNewIndex("studentProfId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexsaveFn, err := GetLazyCallBackFunc(LazyCallBackFnSaveIndex)
	if err != nil {
		log.Fatal(err)

	}
	idxFP := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexID)
	indexLazyObj := lazywriter.LazyCacheObject{
		FileName:      idxFP,
		Identifier:    i.IndexID,
		InterfaceData: i,
		SaveFn:        indexsaveFn,
	}

	IndexLazyObjHolder.SetNoExpiration(i.IndexID, indexLazyObj)

	// saveFn, err := GetLazyCallBackFunc(LazyCallBackFnAppendBucket)
	// if err != nil {
	// 	log.Fatal(err)
	// }
	// appendBucketPath := filepath.Join(db.DBPath, b.BucketPath)
	// lazyObj := lazywriter.LazyCacheObject{
	// 	FileName:      appendBucketPath,
	// 	Identifier:    b.BucketID,
	// 	InterfaceData: nil,
	// 	SaveFn:        saveFn,
	// }
	// AppendLazyObjHolder.SetNoExpiration(b.BucketID, lazyObj)

	data, _ := sjson.Set("", "name", "vijay")
	data, _ = sjson.Set(data, "studentProfId", "10013")
	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 24)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentProfObj := gjson.Parse(data)

	err = SaveDataInFDB("myfdb", i.IndexID, &studentProfObj)
	if err != nil {
		log.Fatal(err)
	}

	time.Sleep(time.Second * 15)

}
func TestSaveDataInAppendBucket(t *testing.T) {
	Init(false, false, false, "")
	// for encryption
	// Init(true, false, false, "")

	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("appendBucket", false, &Bucket{})
	b.SetBucketType(BucketTypeAppend)

	// loggermdl.LogDebug(infileIndexExam)
	i, err := db.GetNewIndex("studentProfId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "name", "ajay")
	data, _ = sjson.Set(data, "studentProfId", "10013")
	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 24)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentProfObj := gjson.Parse(data)

	// data2, _ := sjson.Set("", "name", "ajay")
	// data2, _ = sjson.Set(data2, "studentId", 10013)
	// data2, _ = sjson.Set(data2, "examName", "unit2")
	// data2, _ = sjson.Set(data2, "totalQuestion", 50)
	// data2, _ = sjson.Set(data2, "marks", 26)
	// data2, _ = sjson.Set(data2, "fileType", "Exam")
	// studentExamObj := gjson.Parse(data2)
	err = SaveDataInFDB("myfdb", i.IndexID, &studentProfObj)
	if err != nil {
		log.Fatal(err)
	}
	// err = SaveDataInAppendBucket("myfdb", i.IndexID, &studentExamObj)
	// if err != nil {
	// 	log.Fatal(err)
	// }
	// indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	// err = LogFDBIndexFile(indexFilePath, i)
	// if err != nil {
	// 	log.Fatal(err)
	// }
}

func TestLoadFDBEncryptedData(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("appendBucket", false, &Bucket{})
	b.SetBucketType(BucketTypePack)

	infileIndexProfile := filepack.InFileIndex{
		FileType: "Profile",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "class",
				Query:     "class",
			},
		},
	}
	infileIndexExam := filepack.InFileIndex{
		FileType: "Exam",
		IndexFields: []filepack.InFileIndexField{
			filepack.InFileIndexField{
				FieldName: "marks",
				Query:     "marks",
			},
		},
	}
	// loggermdl.LogDebug(infileIndexExam)
	b.SetInFileIndex(infileIndexProfile)
	b.SetInFileIndex(infileIndexExam)
	i, err := db.GetNewIndex("studentProfId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}

	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	err = i.indexObj.View(func(tx *buntdb.Tx) error {
		return tx.Ascend(i.IndexID, func(key, value string) bool {
			loggermdl.LogDebug("key ", key, "value", value)
			return true
		})
	})

	if err != nil {
		log.Fatal(err)
	}
}
func TestSaveMediaInFDB(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackMixed", false, &Bucket{})
	b.SetBucketType(BucketTypeMixed)

	// infileIndexProfile := filepack.InFileIndex{}
	i, err := db.GetNewIndex("stdId", false)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "contentType", "image/png")
	data, _ = sjson.Set(data, "name", "abc")
	data, _ = sjson.Set(data, "fileType", "Asset")
	rs := gjson.Parse(data)

	filePath := "/home/vivekn/Pictures/pqr.png"

	dataByte, err := filemdl.ReadFile(filePath)
	if err != nil {
		log.Fatal(err)
	}
	// dataByte = []byte("hello")

	path, err := SaveMediaInFDB("myfdb", i.IndexID, dataByte, &rs)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogError("path: ", path)
	loggermdl.LogError("recordID: ", filepath.Base(path))
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestGetMediaFromFDB(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackMixed", false, &Bucket{})
	b.SetBucketType(BucketTypeMixed)

	// infileIndexProfile := filepack.InFileIndex{}
	i, err := db.GetNewIndex("stdId", false)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}
	// dataByte = []byte("hello")
	rowID := "4385857390816664"
	// please provide record id
	recordID := "1X41JjhjwLJMRAweaH46wmKGJHR"
	data, fileMeta, err := GetMediaFromFDB("myfdb", i.IndexID, rowID, recordID)
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogDebug(len(data))
	loggermdl.LogDebug(fileMeta)
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestUpdateMediaInFDB(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackMixed", false, &Bucket{})
	b.SetBucketType(BucketTypeMixed)

	// infileIndexProfile := filepack.InFileIndex{}
	i, err := db.GetNewIndex("stdId", false)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "contentType", "image/png")
	data, _ = sjson.Set(data, "name", "abc")
	data, _ = sjson.Set(data, "fileType", "Asset")
	rs := gjson.Parse(data)

	filePath := "/home/vivekn/Pictures/abc.png"

	dataByte, err := filemdl.ReadFile(filePath)
	if err != nil {
		log.Fatal(err)
	}
	// dataByte = []byte("hello")
	recordID := "1W1L8MwcroqYRyqRRKOqLJgPxRN"
	path, err := UpdateMediaInFDB("myfdb", i.IndexID, recordID, dataByte, &rs)
	if err != nil {
		log.Fatal(err)
	}

	loggermdl.LogError(path)
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestUpsertMediaInFDB(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("bucketPackMixed", false, &Bucket{})
	b.SetBucketType(BucketTypeMixed)

	// infileIndexProfile := filepack.InFileIndex{}
	i, err := db.GetNewIndex("stdId", false)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	data, _ := sjson.Set("", "contentType", "image/png")
	data, _ = sjson.Set(data, "name", "abc")
	data, _ = sjson.Set(data, "fileType", "Asset")
	rs := gjson.Parse(data)

	filePath := "/home/vivekn/Pictures/1.png"

	dataByte, err := filemdl.ReadFile(filePath)
	if err != nil {
		log.Fatal(err)
	}
	// dataByte = []byte("hello")
	recordID := "1W1L8MwcroqYRyqRRKOqLJgPxRN"
	path, err := UpsertMediaInFDB("myfdb", i.IndexID, recordID, dataByte, &rs)
	if err != nil {
		log.Fatal(err)
	}

	loggermdl.LogError(path)
	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestLazySaveInNormalBucket(t *testing.T) {
	Init(false, false, true, "")
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("normalLazy", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)
	b.EnableLazyWrite(true)
	// loggermdl.LogDebug(infileIndexExam)
	i, err := db.GetNewIndex("studentProfId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexsaveFn, err := GetLazyCallBackFunc(LazyCallBackFnSaveIndex)
	if err != nil {
		log.Fatal(err)

	}
	idxFP := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexID)
	indexLazyObj := lazywriter.LazyCacheObject{
		FileName:      idxFP,
		Identifier:    i.IndexID,
		InterfaceData: i,
		SaveFn:        indexsaveFn,
	}

	IndexLazyObjHolder.SetNoExpiration(i.IndexID, indexLazyObj)

	// saveFn, err := GetLazyCallBackFunc(LazyCallBackFnAppendBucket)
	// if err != nil {
	// 	log.Fatal(err)
	// }
	// appendBucketPath := filepath.Join(db.DBPath, b.BucketPath)
	// lazyObj := lazywriter.LazyCacheObject{
	// 	FileName:      appendBucketPath,
	// 	Identifier:    b.BucketID,
	// 	InterfaceData: nil,
	// 	SaveFn:        saveFn,
	// }
	// AppendLazyObjHolder.SetNoExpiration(b.BucketID, lazyObj)

	data, _ := sjson.Set("", "name", "ajay")
	data, _ = sjson.Set(data, "studentProfId", "10013")
	data, _ = sjson.Set(data, "class", "TY_MCA")
	data, _ = sjson.Set(data, "age", 24)
	data, _ = sjson.Set(data, "fileType", "Profile")
	studentProfObj := gjson.Parse(data)

	err = SaveDataInFDB("myfdb", i.IndexID, &studentProfObj)
	if err != nil {
		log.Fatal(err)
	}
	data, _ = sjson.Set(data, "studentProfId", "10014")

	nesStd := gjson.Parse(data)
	err = SaveDataInFDB("myfdb", i.IndexID, &nesStd)
	if err != nil {
		log.Fatal(err)
	}
	queries := []string{`#[name=="ajay"]`}
	res, err := ReadDataFromFDB("myfdb", i.IndexID, nil, queries, []string{})
	if err != nil {
		log.Fatal(err)
	}
	loggermdl.LogError("res", res)
	datatoUpdateStr, _ := sjson.Set("", "newField", "val")
	datatoUpdate := gjson.Parse(datatoUpdateStr)
	updatedRes, errList := UpdateDataInFDB("myfdb", i.IndexID, &datatoUpdate, queries, []string{})
	// if err != nil {
	// 	log.Fatal(err)
	// }
	loggermdl.LogError("updatedRes", updatedRes)
	loggermdl.LogError("errList", errList)
	// delRes, errList := DeleteDataFromFDB("myfdb", i.IndexID, &datatoUpdate, queries, []string{})
	// loggermdl.LogError("delRes", delRes)
	// loggermdl.LogError("del errList", errList)
	time.Sleep(time.Second * 10)
}

func TestExportDataFromNormalBucket(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	zipExporter := ZipExporter{
		DestPath:      "/home/vivekn/fdb_data/dest",
		IndexID:       i.IndexID,
		MigrationType: MigrationTypeUpdate,
		Queries:       []string{},
		FdbName:       db.DBName,
	}

	err = zipExporter.DataExport()
	if err != nil {
		log.Fatal(err)
	}
}
func TestImportDataIntoFDB(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/importedFDB", "importedFDB", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	zipImporter := ZipImporter{
		Data:       nil,
		SourcePath: "/home/vivekn/fdb_data/dest/studentId",
		FdbName:    db.DBName,
		IndexID:    i.IndexID,
	}
	err = zipImporter.DataImport()
	// err := ImportZip("myfdb", "/home/vivekn/fdb_data/dest/stdId")
	if err != nil {
		loggermdl.LogError(err)
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}

func TestExportDataAsFiles(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/myfdb", "myfdb", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)
	err = LoadFDBIndexFromFile(indexFilePath, db, i.IndexID)
	if err != nil {
		log.Fatal(err)
	}

	exporter := FileExporter{
		DestPath:      "/home/vivekn/fdb_data/dest",
		IndexID:       i.IndexID,
		MigrationType: MigrationTypeUpdate,
		Queries:       []string{},
		FdbName:       db.DBName,
	}

	err = exporter.DataExport()
	if err != nil {
		log.Fatal(err)
	}
}
func TestImportDataFromFile(t *testing.T) {
	db, err := CreateFDBInstance("/home/vivekn/fdb_data/importedFDB", "importedFDB", false)
	if err != nil {
		log.Fatal("CreateFDBInstance = ", err)
	}
	// step 1:  create bucket
	b := db.GetNewBucket("Simple", false, &Bucket{})
	b.SetBucketType(BucketTypeSimple)

	i, err := db.GetNewIndex("studentId", true)
	if err != nil {
		log.Fatal(err)
	}
	i.SetBucket(b)
	fields := []IndexField{
		IndexField{
			FieldName: "name",
			Query:     "name",
		},
	}
	i.SetFields(fields...)
	// step 2:  create index
	err = db.CreateIndex(i)
	if err != nil {
		log.Fatal(err)
	}
	fileImporter := FileImporter{
		Data:       nil,
		SourcePath: "/home/vivekn/fdb_data/dest/studentId",
		FdbName:    db.DBName,
		IndexID:    i.IndexID,
	}
	err = fileImporter.DataImport()
	// err := ImportZip("myfdb", "/home/vivekn/fdb_data/dest/stdId")
	if err != nil {
		loggermdl.LogError(err)
		log.Fatal(err)
	}
	indexFilePath := filepath.Join(db.DBPath, INDEXFOLDER, i.IndexNameQuery)

	err = LogFDBIndexFile(indexFilePath, i)
	if err != nil {
		log.Fatal(err)
	}
}
